/** Displays a partially transparent image
 *  And fades between 4 different images.
 *
 *  @author Vitus Lorenz-Meyer
 *  @version 1.0 $Rev$
 */

package Control;
import util.*;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class ClPoser extends JLabel{
	/** Double Buffer Graphics */
	Graphics2D 	g_buffer;
	/** Double Buffer */
	BufferedImage	i_buffer;
	/** Images to display */
	private Image[] mimImages = new Image[4];
	/** Alpha of Foreground pic */
	private AlphaComposite moForeComposite = AlphaComposite.SrcOver;
	/** Alpha of Background pic */
	private AlphaComposite moBackComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,0);
	/** the current pic to display with foreground alpha */
	private int miForePic = 0;
  	/** the current pic to display with background alpha */
	private int miBackPic = 0;
	/** Width of Images to display */
	private int imgWidth;
	/** Height of Images to display */
	private int imgHeight;

  	/** Contructor
  	 *  @param piFadeTime - the time to fade in sec
  	 *  @param piFPS - FPS to use */
	public ClPoser(final int piFadeTime,final int piFPS) throws EXXYError {
    		super();

		final java.util.Random r = new java.util.Random();
		miForePic = r.nextInt(4);
		miBackPic = r.nextInt(4);

    		final java.util.Timer timer = new java.util.Timer(true);      // Timer to run clock

    		long frameInterval = 1000/piFPS;

    		loadImages();
		setHorizontalAlignment(CENTER);
		setPreferredSize(new Dimension(imgWidth+10,imgHeight+10));
		setOpaque(false);


		timer.schedule(new java.util.TimerTask()
		{
                	private int count = 0;
                	private int maxCount = piFadeTime*piFPS;
                	float alphaStep = 1.0f/maxCount;

                 	public void run()
                      	{
                        	repaint();      // Repaint the image
				// Update alpha composite for next frame
				if(count == maxCount){
					count=0;
					miBackPic = miForePic;
					miForePic = r.nextInt(4);
	                         }
				//count up
				count++;
				try{
                       			moForeComposite = AlphaComposite.getInstance(
                              			AlphaComposite.SRC_OVER,count*alphaStep );
                              		moBackComposite = AlphaComposite.getInstance(
	                              		AlphaComposite.SRC_OVER,(maxCount-count)*alphaStep );
	                        }catch (Exception e){}
			}
		},
		0,                       // ...starting now
                frameInterval);          // Repaint interval

		setVisible(true);


    	}

    	/** loads all Images and waits till they are completely loaded, also creates DoubleBuffer (offscreen Image) */
	private void loadImages() throws EXXYError{
    		MediaTracker tracker = new MediaTracker(this);
    		byte i;
    		Image images[] = new Image[4];
    		Toolkit tk = Toolkit.getDefaultToolkit();
		try {
	    		for (i=0;i<4;i++){
    				mimImages[i] = tk.getImage("./res/about"+i+".png");
    				tracker.addImage(mimImages[i],i);
    			}
      			tracker.waitForAll() ;
      		}
    		catch (Exception e) { throw new EInternalError("Error");  }
    		imgWidth = mimImages[0].getWidth(this);
    		imgHeight = mimImages[0].getHeight(this);

		//create the buffer
		try{
			i_buffer = new BufferedImage(imgWidth,imgHeight,BufferedImage.TYPE_INT_ARGB);
			g_buffer = (Graphics2D)i_buffer.getGraphics();
			g_buffer.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_SPEED);
		}catch (Exception e) { e.printStackTrace(); }

    	}

    	/** paints this window, overriden from JComponent
    	 *  @param g - the Graphics Object to paint on
     	 *  @see javax.swing.JComponent */
	public void paint(Graphics g) {
		//paint everything to the buffer first
		try{

			g_buffer.setComposite(moForeComposite);
			g_buffer.drawImage(mimImages[miForePic], 0, 0, null);
			g_buffer.setComposite(moBackComposite);
			g_buffer.drawImage(mimImages[miBackPic], 0, 0, null);

			//draw the buffer finally

			g.drawImage(i_buffer,(getWidth()/2)-(imgWidth/2),5,null);

		}catch (Exception e) { e.printStackTrace(); }
	}
	/** updates (paints) this window, overriden from JComponent
    	 *  @param g - the Graphics Object to paint on
     	 *  @see javax.swing.JComponent */
	public void update (Graphics g)
	{
   		// Just call paint to avoid flickering
   		paint(g);
	}

}
