/** Displays a partially transparent image as a splash screen.
 *  And fades between 4 different images.
 *
 *  @author Vitus Lorenz-Meyer
 *  @version 1.2 $Rev: 257 $
 */

package Control;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class ClSplash extends Window {
	/** Double Buffer Graphics */
	Graphics2D 	g_buffer;
	/** Double Buffer */
	BufferedImage	i_buffer;

	/** Image of Background */
	private Image mimBack;
	/** Images to display */
	private Image mimImage;
	/** Alpha of pic */
	private AlphaComposite moComposite = AlphaComposite.getInstance(
                              			AlphaComposite.SRC_OVER,0);

	/** Contructor
  	 *  @param piFadeTime - the time to fade in,out and between in sec
  	 *  @param piFPS - FPS to use */
	public ClSplash(final int piFadeTime,final int piFPS) {
    		super(new Frame());

    		final Timer timer = new Timer(true);      // Timer to run clock

    		long frameInterval = 1000/piFPS;

    		loadImage();

		timer.schedule(new TimerTask()
		{
                	private int count = 1;
                	private int maxCount = piFadeTime*piFPS;
                	float alphaStep = 1.0f/maxCount;
                	private int countDelta = 1;

                 	public void run()
                      	{
                        	repaint();      // Repaint the image
				// Update alpha composite for next frame depending on state
				if(count == (maxCount+18)) countDelta=-1;
				else if (count == 0) timer.cancel();

				//count up
				count += countDelta;
				try{
                       			moComposite = AlphaComposite.getInstance(
                              			AlphaComposite.SRC_OVER, Math.min(1.0f,count*alphaStep));
                              	}catch (Exception e){}
			}
		},
		0,                       // ...starting now
                frameInterval);          // Repaint interval

		setVisible(true);
    		show();

    	}

    	/** loads a random logo and the background and waits till its completely loaded */
	private void loadImage() {
    		MediaTracker tracker = new MediaTracker(this);
    		Toolkit tk = Toolkit.getDefaultToolkit();
		Random r = new Random();

		try {
    			mimImage = tk.getImage("./res/logo"+(r.nextInt(4))+".png");
    			tracker.addImage(mimImage,0);
			tracker.waitForAll() ;
      		}
    		catch (Exception e) { e.printStackTrace(); }
    		int imgWidth = mimImage.getWidth(this);
    		int imgHeight = mimImage.getHeight(this);

		//create the buffer
		try{
			i_buffer = new BufferedImage(imgWidth,imgHeight,BufferedImage.TYPE_INT_ARGB);
			g_buffer = (Graphics2D)i_buffer.getGraphics();
			g_buffer.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_SPEED);
		}catch (Exception e) { e.printStackTrace(); }


		//load the Back Image
    		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((d.width - imgWidth) / 2, (d.height - imgHeight) / 3, imgWidth, imgHeight);
		try { mimBack = new Robot().createScreenCapture(getBounds()); }
		catch (AWTException e) {e.printStackTrace(); }

    	}

    	/** paints this window, overriden from JComponent
    	 *  @param g - the Graphics Object to paint on
     	 *  @see javax.swing.JComponent */
	public void paint(Graphics g) {
		//paint everything to the buffer first
		try{
			g_buffer.setComposite(AlphaComposite.SrcOver);
			g_buffer.drawImage(mimBack, 0, 0, null);

			g_buffer.setComposite(moComposite);
			g_buffer.drawImage(mimImage, 0, 0, null);

			//draw the buffer finally to the screen
			g.drawImage(i_buffer,0,0,null);
		}catch (Exception e) { e.printStackTrace(); }
	}
	/** updates (paints) this window, overriden from JComponent
    	 *  @param g - the Graphics Object to paint on
     	 *  @see javax.swing.JComponent */
	public void update (Graphics g)
	{
   		// Just call paint to avoid flickering
   		paint(g);
	}

}
