/** Main class of Xxy.
 *  Defines the main frame as well as most of the program logic, all the menus and the toolbar.
 *  Private classes are subclasses of AbstractAction for wiring together the menu entry and
 *  the toolbar button
 *
 *  @author Vitus Lorenz-Meyer
 *  @version 1.2 $Rev: 261 $
 */

package Control;
import Control.Editor.*;
import util.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.beans.*;
//import com.l2fprod.gui.plaf.skin.*;

public class ClXxy extends JFrame implements ActionListener{

	/** Version */
	public static final String mstrVersion = "Xxy Version 0.4, build 30";

	/** the main Frame (i.e. this) */
	private JFrame mwTopWnd;

	/** the Resource Bundle of the selected language*/
	public static ResourceBundle moResBundle = null;

	/** the selected tool as defined in IControl
	 *  @see IControl */
	private byte miSelectedTool;

	/** button-array for tool selection needed for de/selecting the buttons */
	private JToggleButton[] maToolButtons;

	/** MenuItem-array for tool selection needed for de/selecting the buttons */
	private JRadioButtonMenuItem[] maToolMenuItems;

	/** the Properties toolbar */
	private ClEditorWidget moEditorWidget;

	/** the normal toolbar */
	private JToolBar moToolBar;

	/** main client area Panel */
	private JDesktopPane moClientArea;

	/** status TextField */
	private JTextField mtStatusText;

	/** toggle button for grid vivisble */
	private JToggleButton moGridVisibleButton;

	/** Menu Entry for for grid vivisble */
	private JCheckBoxMenuItem moGridVisibleMenuItem;

	/** Array for Things that need to be deativated if no files open */
	private Vector mvDocumentDependentActions;

	/** grid size slider */
	private JSlider moGridSizeSlider;

	/** Constructor of ClXxy
	 *  @param pbNoSplash - display no Splash screen
	 *  @param pstrArgs - String Array with files to open */
	public ClXxy(boolean pbNoSlpash,String[] pstrArgs){
		super("Xxy");

		int i;
		mwTopWnd = this;

		ClSplash splash = null;
		System.setProperty("awt.image.incrementaldraw","false");

	        	try {
				UIManager.setLookAndFeel(
					UIManager.getCrossPlatformLookAndFeelClassName());
			} catch (Exception e) { }

		if (!pbNoSlpash){
			try {
				splash = new ClSplash(2,36);
				Thread.sleep(1050*2*2+1000);
				splash.dispose();

			}catch (Exception e) { }
		}
		//create main Window and load Language

	        //this = new JFrame;
        	this.setIconImage(Toolkit.getDefaultToolkit().getImage("./res/window.gif"));

        	//some var init
        	miSelectedTool=IControl.SELECTION_TOOL;

		try{
			 moResBundle = ResourceBundle.getBundle("Lang.Xxy",Locale.getDefault());
		}catch (Exception e){ moResBundle = new ResourceBundle() {
					public Locale getLocale(){ return null;}
					protected Object handleGetObject(String key){ return null;}
					public Enumeration getKeys(){ return null;}
					};
			}
		this.getAccessibleContext().setAccessibleDescription(_("An editor for Xy-Pic Code for LaTeX"));

	        try{ CreateMenuAndToolBars(); }
	        catch(EXXYError ei){
			eMsg(_("Internal Error while initializing menus and toolbar"),ei,true);
			System.exit(1);
		}
		catch(Exception ex){
			eMsg(_("Error while initializing menus and toolbar"),ex,false);
			System.exit(1);
		}

	        //open the req. files
	        try{ if (pstrArgs.length>0){
			for (byte j=0;j<pstrArgs.length;j++){
				if (pstrArgs[j]!=null){
					ClXxy_childWnd xxcw = new ClXxy_childWnd(moEditorWidget,mtStatusText,this,new File(pstrArgs[j]));
        		    		moClientArea.add(xxcw);
            				if (ClXxy_childWnd.openFrameCount==1){
            					Object[] ar = mvDocumentDependentActions.toArray();
						for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(true);
						((JSlider)ar[i++]).setEnabled(true);
						((JLabel)ar[i++]).setEnabled(true);
					}
        	    			addIFL(xxcw);
            				try { xxcw.setSelected(true); }
    					catch (java.beans.PropertyVetoException exc) {}
					xxcw.show();
				}
			}
		}
		}catch(EXXYError ei){
			eMsg(_("Internal Error while opening the requested files"),ei,true);
		}
		catch(Exception ex){
			eMsg(_("Error while opening the requested files"),ex,false);
		}

		//Display the window
	        this.pack();
        	this.setSize(900, 600);

	        this.setVisible(true);

	}
	/** Creates the Menus and the toolbar at startup and anytime the language is changed
	 *  @throws an XXYError if something went wrong  */
	private void CreateMenuAndToolBars() throws EXXYError{
		byte i;
		mvDocumentDependentActions = new Vector();

		//reset main Frame
		this.getContentPane().removeAll();

		maToolButtons = new JToggleButton[3];
		maToolMenuItems = new JRadioButtonMenuItem[3];

		//ToolBar
		JToolBar moToolBar = new JToolBar();
		moToolBar.setFloatable(false);


		//menu
        	JMenuBar moMenuBar = new JMenuBar();


		//top level entries
			JMenu myMenuFile = new JMenu(_("File"));
			myMenuFile.setMnemonic( ((int) (_("MenuFile_Mnemonic")).charAt(0) ) );

			JMenu myMenuEdit = new JMenu((String)_("Edit"));
			myMenuEdit.setMnemonic( ((int) (_("MenuEdit_Mnemonic")).charAt(0) ) );

			JMenu myMenuOption = new JMenu((String)_("Option"));
    			myMenuOption.setMnemonic('O');

	    		JMenu myMenuHelp = new JMenu((String)_("Help"));
    			myMenuHelp.setMnemonic( 'H' );

			moMenuBar.add(myMenuFile);
			moMenuBar.add(myMenuEdit);
			moMenuBar.add(myMenuOption);
			moMenuBar.add(myMenuHelp);

		//sub entries and moToolBar buttons

        		//File-menu

				final Action vExitAction;

	        		vExitAction = new ExitAction();
        			Action[] Actions = {new NewDiaAction(),new OpenDiaAction(),new CloseDiaAction(),null,
        					    new SaveDiaAction(),new SaveAsAction(),null,vExitAction};

				mvDocumentDependentActions.addElement(Actions[2]);
				mvDocumentDependentActions.addElement(Actions[4]);
				mvDocumentDependentActions.addElement(Actions[5]);

				loadActions(Actions,myMenuFile,moToolBar);

			//Edit Menu
				moToolBar.addSeparator();

				//Paste,Copy
				Action[] Actions6 = {new CopyAction(),new PasteAction(),null};
				loadActions(Actions6,myMenuEdit,moToolBar);
				moToolBar.addSeparator();
				mvDocumentDependentActions.addElement(Actions6[0]);


				//Tools
		    		JMenuItem myMenuItem;
		    		JToggleButton b;
				ButtonGroup myGroup = new ButtonGroup();
				Action[] Actions5 = {new SelectAction(),new CreateLabelAction(),new CreateArrowAction()};
				mvDocumentDependentActions.addElement(Actions5[0]);
				mvDocumentDependentActions.addElement(Actions5[1]);
				mvDocumentDependentActions.addElement(Actions5[2]);

				for (i=0;i<3;i++){
					myMenuItem = new JRadioButtonMenuItem(Actions5[i]);
					b = new JToggleButton(Actions5[i]);
					b.setText("");
					b.setMnemonic(0);

					if (i==miSelectedTool){ myMenuItem.setSelected(true); b.setSelected(true);}

					myGroup.add(myMenuItem);
					myMenuEdit.add(myMenuItem);
        				moToolBar.add(b);

        				maToolButtons[i]=b;
        				maToolMenuItems[i]=(JRadioButtonMenuItem)myMenuItem;

				}
				//delete
				Action[] Actions2 = {null,new DeleteAction()};
				loadActions(Actions2,myMenuEdit,moToolBar);
				mvDocumentDependentActions.addElement(Actions2[1]);


			//Options Menu

				//show grid

					moToolBar.addSeparator();

					Action a = new ShowGridAction();
					moGridVisibleButton = new JToggleButton(a);
	        			moGridVisibleButton.setText("");
        				moGridVisibleButton.setMnemonic(0);
        				moGridVisibleMenuItem = new JCheckBoxMenuItem(a);
        				myMenuOption.add(moGridVisibleMenuItem);
        				moToolBar.add(moGridVisibleButton);
        				mvDocumentDependentActions.addElement(a);


				//L&F

					UIManager.LookAndFeelInfo[] LAFInfo = UIManager.getInstalledLookAndFeels();

					JMenu submenu = new JMenu("Look&Feel");
					submenu.setMnemonic(KeyEvent.VK_F);

					myGroup = new ButtonGroup();
					for (i=0;i<LAFInfo.length;i++){
						myMenuItem = new JRadioButtonMenuItem(LAFInfo[i].getName());
						myMenuItem.setAccelerator(KeyStroke.getKeyStroke(0x31+i,InputEvent.CTRL_MASK | InputEvent.ALT_MASK));
			        		myMenuItem.setActionCommand("LF_"+i);
        					myMenuItem.addActionListener(this);
						submenu.add(myMenuItem);
						if (UIManager.getLookAndFeel().getName().equals(LAFInfo[i].getName()))
							myMenuItem.setSelected(true);

						myGroup.add(myMenuItem);

					}
					/*
					submenu.addSeparator();
					File fdir = new File("./skins");
					File[] files = fdir.listFiles();
					for (i=0;i<files.length;i++){
						myMenuItem = new JRadioButtonMenuItem(files[i].getName().substring(0,(files[i].getName().length()-8)));
						myMenuItem.setActionCommand("SLF_"+files[i].getName());
        					myMenuItem.addActionListener(this);
						submenu.add(myMenuItem);
						//if (UIManager.getLookAndFeel().getName().equals(LAFInfo[i].getName()))
						//	myMenuItem.setSelected(true);

						myGroup.add(myMenuItem);
					}
					*/

					myMenuOption.add(submenu);

				//Language
					submenu = new JMenu((String)_("Language"));
					submenu.setMnemonic(KeyEvent.VK_L);

					myGroup = new ButtonGroup();

					//en_US
					String loc = "en_US";
					myMenuItem = new JRadioButtonMenuItem(loc);
					myMenuItem.setActionCommand("Go_"+loc);
					myMenuItem.addActionListener(this);
					if (moResBundle.getLocale()==null) myMenuItem.setSelected(true);

					submenu.add(myMenuItem);
					myGroup.add(myMenuItem);

					//files in Lang/
					File fdir = new File("./Lang");
					File[] files = fdir.listFiles();
					for (i=0;i<files.length;i++){
						if (files[i].getName().endsWith(".properties")){
							loc = files[i].getName().substring(4,(files[i].getName().length()-11));
							myMenuItem = new JRadioButtonMenuItem(loc);
							myMenuItem.setActionCommand("Go_"+loc);
							myMenuItem.addActionListener(this);
							try{ if ( loc.equals(moResBundle.getLocale().toString()) )
								myMenuItem.setSelected(true);
							}catch(Exception ex){}
							submenu.add(myMenuItem);
							myGroup.add(myMenuItem);
						}
					}


					myMenuOption.add(submenu);
			//Help Menu

    				moToolBar.addSeparator();
				Action[] Actions3 = {new InfoAction()};
				loadActions(Actions3,myMenuHelp,moToolBar);



		//create floatable Properties moToolBar
		moEditorWidget = new ClEditorWidget(this);
		moEditorWidget.setFloatable(false);

		//Status Bar
		JPanel statusPane = new JPanel(new GridLayout(1, 1));
		statusPane.setPreferredSize(new Dimension(700,18));

		mtStatusText = new JTextField(_("ready"));
        	mtStatusText.setPreferredSize(new Dimension(700,18));
		mtStatusText.setEditable(false);
		mtStatusText.setBorder(BorderFactory.createLoweredBevelBorder());

		statusPane.add(mtStatusText);

		//main client area
		moClientArea = new JDesktopPane();
		moClientArea.setBorder(BorderFactory.createLoweredBevelBorder());
		moClientArea.setBackground(Color.gray);
		moClientArea.setPreferredSize(new Dimension(1000,1000));

		//Grid Size Slider
		moGridSizeSlider = new JSlider(JSlider.HORIZONTAL, 50, 200, 100);
		moGridSizeSlider.setPreferredSize(new Dimension(400,20));
		moGridSizeSlider.addChangeListener(
            		new ChangeListener() {
                		public void stateChanged(ChangeEvent e){
                    			try{ ((ClXxy_childWnd)moClientArea.getSelectedFrame()).setGridSize((int)((JSlider)e.getSource()).getValue());
                    			}catch (Exception ex) { }
                    		}
            		}
        	);

        	moGridSizeSlider.setMajorTickSpacing(25);
	        moGridSizeSlider.setMinorTickSpacing(5);
        	moGridSizeSlider.setPaintTicks(true);
		mvDocumentDependentActions.addElement(moGridSizeSlider);

		moToolBar.addSeparator();
		moToolBar.add(moGridSizeSlider);

		JLabel myLabel = new JLabel(createIcon("Zoom"), JLabel.LEADING);
		mvDocumentDependentActions.addElement(myLabel);
		myLabel.setPreferredSize(new Dimension(500,20));

		moToolBar.add(myLabel);


		//Set the menu bar and add moToolBar
        	JPanel pane = new JPanel(new BorderLayout());
		JPanel pane2 = new JPanel(new BorderLayout());

		pane2.add(moEditorWidget, BorderLayout.SOUTH);
		pane2.add(moClientArea, BorderLayout.CENTER);
		pane.add(moToolBar, BorderLayout.NORTH);
		pane.add(pane2, BorderLayout.CENTER);

		this.setJMenuBar(moMenuBar);
		this.getContentPane().add(pane, BorderLayout.CENTER);
		this.getContentPane().add(statusPane, BorderLayout.SOUTH);

		this.addWindowListener(
       			new WindowAdapter() {
				public void windowClosing(WindowEvent event)
				{	//call the method of ExitAction class
					vExitAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,null));
				}
		});
		//deactivate thingies which are Doc dependet ... save,close...
		if (ClXxy_childWnd.openFrameCount==0){
			Object[] ar = mvDocumentDependentActions.toArray();
			for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(false);
			((JSlider)ar[i++]).setEnabled(false);
			((JLabel)ar[i++]).setEnabled(false);

		}
		//select nothing
		moEditorWidget.selectionChanged(null,PL.IPL.NOTHING);

	}

	/** called when L&F or language is changed
	 *  @param ActionEvent - an event assiocated with the firing Entry */
	public void actionPerformed(ActionEvent e) {
 		String com = e.getActionCommand();

 		if (com.startsWith("Go_")){		//change Language
 			int i;
 			String lang="",country="",variant="";
			this.setVisible(false);

			String loc = com.substring(3,com.length());
 			//set resource bundle to new anonymous extended resourceBundle class, coz ResB is abstract and cannot be instantiated
 			if (loc.equals("en_US")) moResBundle = new ResourceBundle() {
					public Locale getLocale(){ return null;}
					protected Object handleGetObject(String key){ return null;}
					public Enumeration getKeys(){ return null;}
					};
 			else{
 				try{
					lang = loc.substring(0,2);
 					country = loc.substring(3,5);
 					variant = loc.substring(6,loc.length());
 				}catch(Exception ex){ }

 				try{ moResBundle = ResourceBundle.getBundle("Lang.Xxy",new Locale(lang,country,variant)); }
				catch (Exception exc){ }
			}

			//save old InternalFrames before destroying old one

			JInternalFrame[] aJIFrames = moClientArea.getAllFrames();
			JInternalFrame jif = moClientArea.getSelectedFrame();
			moClientArea.removeAll();

			try{ CreateMenuAndToolBars(); }
	     		catch(EXXYError ei){
				eMsg(_("Internal Error while initializing menus and toolbar"),ei,true);
				System.exit(1);
			}
			catch(Exception ex){
				eMsg(_("Error while initializing menus and toolbar"),ex,false);
				System.exit(1);
			}
			this.invalidate();
			this.setVisible(true);

			try{
				for (i=0;i<aJIFrames.length;i++){
					moClientArea.add(aJIFrames[i]);
					//if (jif == aJIFrames[i]) aJIFrames[i].setSelected(true);
					aJIFrames[i].setSelected(false);
					((ClXxy_childWnd)aJIFrames[i]).refresh(moEditorWidget);
				}
	    		} catch (Exception ex) { }
	    		try { jif.setSelected(true); }
	    		catch (java.beans.PropertyVetoException ex) { }



 		}
 		else if (com.startsWith("LF_")){
 			this.setVisible(false);
 			int num = Integer.parseInt(com.substring(3,4));
			UIManager.LookAndFeelInfo[] LAFInfo = UIManager.getInstalledLookAndFeels();
			try{
				UIManager.setLookAndFeel(LAFInfo[num].getClassName());
				SwingUtilities.updateComponentTreeUI(this);
				this.invalidate();

			}catch(Exception ex) {
				eMsg(_("Could not set new L&F"),ex,false);
                	}

			this.setVisible(true);
		}
		/*
		else if (com.startsWith("SLF_")){
 			this.setVisible(false);
 			try{
 				SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePack("./skins/"+com.substring(4,com.length() )));
      				SkinLookAndFeel.enable();
			}catch(Exception ex) { System.err.println("Could'nt set new L&F: "+ex); }

			SwingUtilities.updateComponentTreeUI(this);

			this.invalidate();
			this.setVisible(true);
		}
		*/
	}

	/** called from ClEditorWidget to repaint current DAL */
	public void repaintCurrentDAL(){
		 moClientArea.getSelectedFrame().repaint();
	}

    	//helper funcs

    	/** adds the specified Action to given menu and moToolBar
    	 *  @param paActions - Array of Actions to add
    	 *  @param pmMenu - Menu to add the Actions to
    	 *  @param pToolBar - Toolbar to add the Actions to */
    	private void loadActions(Action[] paActions, JMenu pmMenu, JToolBar pToolBar){
    		byte i;
    		JButton b;
		JMenuItem jmi;

    		for (i=0;i<paActions.length;i++){
			if (paActions[i]==null) pmMenu.addSeparator();
			else{
	       	 		b = new JButton(paActions[i]);
        			b.setText("");
        			b.setMnemonic(0);
        			jmi = new JMenuItem(paActions[i]);
        			pmMenu.add(jmi);
        			pToolBar.add(b);
        		}
        	}
	}

	/** shows an error message
	 *  @param pstrError - the Error String to show
	 *  @param peException - the Exception that occured
	 *  @param pbInternal - if its an InternalError */
	private void eMsg(String pstrError,Exception peException,boolean pbInternal){
		 peException.printStackTrace();
		 JOptionPane.showMessageDialog(this,
			java.text.MessageFormat.format(pstrError+":\n{0}", new Object[] { peException.toString() }),
			(pbInternal)?_("Internal Error!"):_("Error!"),
			JOptionPane.ERROR_MESSAGE );
	}
    	/** Returns an ImageIcon, or null if the path was invalid.
    	 *  @param pstrImageName - Name of the Image without trailing .gif
    	 *  @return newly created ImageIcon */
    	public static ImageIcon createIcon(String pstrImageName) {
		ImageIcon ic = null;
		try {ic = new ImageIcon("./res/"+pstrImageName+"16.gif"); }
		catch(Exception e){ ic = new ImageIcon("./res/About16.gif"); }
		return ic;
        }

        /** returns the localized String - only for convenience so that _("String") works within this class
         *  @param pstrGid - the english string to localize
	 *  @return the localized version of pstrGid */
        private static String _(String pstrGid){ return gnu.gettext.GettextResource.gettext(moResBundle,pstrGid); }

	/** adds InternalFrameListener for child window change
	 *  @param pChild - Child Window to add FrameListener to
	 *  @throws an XXYError if something went wrong which _cannot_ happen here ;) */
	private void addIFL(ClXxy_childWnd pChild) throws EXXYError{
		pChild.addInternalFrameListener(
				new InternalFrameAdapter() {
					public void internalFrameActivated(InternalFrameEvent e) {
						ClXxy_childWnd jif = (ClXxy_childWnd) e.getInternalFrame();

						try{ 	jif.setTool(miSelectedTool);
							setTitle("Xxy - "+jif.getTitle());	}
						catch(Exception ex){ }
						moGridVisibleButton.setSelected(jif.getGridVisible());
						moGridVisibleMenuItem.setSelected(jif.getGridVisible());
						moGridSizeSlider.setValue(jif.getGridSize());
    					}
    					public void internalFrameClosed(InternalFrameEvent e) {
    						byte i;

    						if (ClXxy_childWnd.openFrameCount==0){
							//deactivate all things, which are doc dependet & select nothing
							Object[] ar = mvDocumentDependentActions.toArray();
							for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(false);
							((JSlider)ar[i++]).setEnabled(false);
							((JLabel)ar[i++]).setEnabled(false);
							setTitle("Xxy");
							moEditorWidget.selectionChanged(null,PL.IPL.NOTHING);
						}
					}

				}

			);
	}

    	//Action classes
    	//File Menu
	/** Action class for "New Diagram"
	 *  @see javax.swing.AbstractAction */
	class NewDiaAction extends AbstractAction {
        	public NewDiaAction() {
            		super((String)_("New"), createIcon("New"));
	        	putValue(SHORT_DESCRIPTION, _("Creates a new Diagram"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_N));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_N,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
			byte i;
			ClXxy_childWnd xxcw=null;

			try{
        	    		xxcw = new ClXxy_childWnd(moEditorWidget,mtStatusText,mwTopWnd);
            			moClientArea.add(xxcw);
            			addIFL(xxcw);
	            		//enable thingies
        	    		if (ClXxy_childWnd.openFrameCount==1){
            				Object[] ar = mvDocumentDependentActions.toArray();
					for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(true);
					((JSlider)ar[i++]).setEnabled(true);
					((JLabel)ar[i++]).setEnabled(true);
				}

        	    		try { xxcw.setSelected(true); }
	    			catch (java.beans.PropertyVetoException ex) { }
				xxcw.show();
			}catch(EXXYError ei){
				eMsg(_("Internal Error while creating new Diagram"),ei,true);
				try{ xxcw.dispose(); }
				catch(Exception e2) { }
			}
			catch(Exception ex){
				eMsg(_("Error while creating new Diagram"),ex,false);
				try{ xxcw.dispose(); }
				catch(Exception e2) { }
			}
            	}
    	}

        /** Action class for "Open Diagram" */
	class OpenDiaAction extends AbstractAction {
        	public OpenDiaAction() {
            		super((String)_("Open"), createIcon("Open"));
	        	putValue(SHORT_DESCRIPTION, _("Opens an existing Diagram"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_O));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_O,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
        		byte i;
        		ClXxy_childWnd xxcw=null;

        		try{
	        		String sXyPicCode=null;
        	    		JFileChooser fc = new JFileChooser();
            			fc.setDialogType( JFileChooser.OPEN_DIALOG );
        			fc.setMultiSelectionEnabled( false );
	        		fc.addChoosableFileFilter(new ClFileFilter("Xxy", _("Xxy files")));

        	    		if(fc.showOpenDialog(mwTopWnd) == JFileChooser.APPROVE_OPTION) {

					xxcw = new ClXxy_childWnd(moEditorWidget,mtStatusText,mwTopWnd,fc.getSelectedFile());
            				moClientArea.add(xxcw);
            				if (ClXxy_childWnd.openFrameCount==1){
	            				Object[] ar = mvDocumentDependentActions.toArray();
						for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(true);
						((JSlider)ar[i++]).setEnabled(true);
						((JLabel)ar[i++]).setEnabled(true);
					}
            				addIFL(xxcw);
	            			try { xxcw.setSelected(true); }
	            			catch (java.beans.PropertyVetoException exc) {}
					xxcw.show();
        	    		}
        	    	}catch(EXXYError ei){
				eMsg(_("Internal Error while opening Diagram"),ei,true);
				try{ xxcw.dispose(); }
				catch(Exception e2) { }
			}
			catch(Exception ex){
				eMsg(_("Error while opening Diagram"),ex,false);
				try{ xxcw.dispose(); }
				catch(Exception e2) { }
			}
        	}
    	}
  	/** Action class for "Close Diagram" */
	class CloseDiaAction extends AbstractAction {
        	public CloseDiaAction() {
            		super((String)_("Close"), createIcon("Close"));
	        	putValue(SHORT_DESCRIPTION, _("Closes the current Diagram"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
        	}
        	public void actionPerformed(ActionEvent e) {
			byte i;
			try{
				try{ moClientArea.getSelectedFrame().setClosed(true); }
				catch (java.beans.PropertyVetoException exc) {  }
				if (ClXxy_childWnd.openFrameCount==0){

					//deactivate all things, which are doc dependet...
					Object[] ar = mvDocumentDependentActions.toArray();
					for (i=0;i<ar.length-2;i++) ((Action)ar[i]).setEnabled(false);
					((JSlider)ar[i++]).setEnabled(false);
					((JLabel)ar[i++]).setEnabled(false);
				}
			}
			catch(Exception ex){
				eMsg(_("Error while closing Diagram"),ex,false);
			}
        	}
    	}
    	/** Action class for "Save Diagram" */
	class SaveDiaAction extends AbstractAction {
        	public SaveDiaAction() {
            		super((String)_("Save"), createIcon("Save"));
	        	putValue(SHORT_DESCRIPTION, _("Saves the current Diagram"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_S,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{ ((ClXxy_childWnd)moClientArea.getSelectedFrame()).save(); }
            		catch(EXXYError ei){
				eMsg(_("Internal Error while saving Diagram"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while saving Diagram"),ex,false);
			}

        	}
    	}
    	/** Action class for "Save Diagram As" */
	class SaveAsAction extends AbstractAction {
        	public SaveAsAction() {
            		super((String)_("Save As"), createIcon("SaveAs"));
	        	putValue(SHORT_DESCRIPTION, _("Saves the current Diagram under a new name"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_A));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_F12,0));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{ ((ClXxy_childWnd)moClientArea.getSelectedFrame()).saveAs(); }
            		catch(EXXYError ei){
				eMsg(_("Internal Error while saving Diagram"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while saving Diagram"),ex,false);
			}
        	}
    	}
    	/** Action class for "Exit" */
	class ExitAction extends AbstractAction {
        	public ExitAction() {
            		super((String)_("Exit"), createIcon("Exit"));
	        	putValue(SHORT_DESCRIPTION, _("Exits Xxy"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_X));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_X,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
            		byte i;

            		try{
	            		boolean bExit=true;
            			JInternalFrame[] aJIFrames = moClientArea.getAllFrames();

				try{
        	    			for (i=0;i<aJIFrames.length;i++) aJIFrames[i].setClosed(true);
            			} catch (PropertyVetoException ex) { return; }

				mwTopWnd.dispose();
				System.exit(0);
			}
			catch(Exception ex){
				eMsg(_("Error while exiting Xxy"),ex,false);
				System.exit(1);
			}
		}


    	}

    	//Edit Menu
    	/** Action class for "Paste" */
	class PasteAction extends AbstractAction {
        	public PasteAction() {
            		super((String)_("Paste"), createIcon("Paste"));
	        	putValue(SHORT_DESCRIPTION, _("Pastes Xy-Pic Code from the Clipboard"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_P));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_V,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
            		ClXxy_childWnd xxcw=null;

        		try {
				java.awt.datatransfer.Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
		        	if (t != null && t.isDataFlavorSupported(java.awt.datatransfer.DataFlavor.stringFlavor)) {
                			String text = (String)t.getTransferData(java.awt.datatransfer.DataFlavor.stringFlavor);
                			if (text.startsWith("\\xy")){
                				xxcw = new ClXxy_childWnd(moEditorWidget,mtStatusText,mwTopWnd,text);
           		 			moClientArea.add(xxcw);
            					addIFL(xxcw);
            					try { xxcw.setSelected(true); }
            					catch (java.beans.PropertyVetoException exc) {}
                				return;
					}
				}
        		}catch(EXXYError ei){
				eMsg(_("Internal Error while pasting from clipboard"),ei,true);
				xxcw.dispose();
				return;
			}
			catch(Exception ex){
				eMsg(_("Error while pasting from clipboard"),ex,false);
				xxcw.dispose();
				return;
			}
        		JOptionPane.showMessageDialog(null, _("no Xy-Pic Code in clipboard"), _("paste error"), JOptionPane.ERROR_MESSAGE);

    		}
    	}
    	/** Action class for "Copy" */
	class CopyAction extends AbstractAction {
        	public CopyAction() {
            		super((String)_("Copy"), createIcon("Copy"));
	        	putValue(SHORT_DESCRIPTION, _("Copys current Diagram as Xy-Pic code to the Clipboard"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
        		putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_C,InputEvent.CTRL_MASK));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{
            			java.awt.datatransfer.StringSelection ss = new java.awt.datatransfer.StringSelection(((ClXxy_childWnd)moClientArea.getSelectedFrame()).XyString());
        			Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ss, null);
        		}
        		catch(EXXYError ei){
				eMsg(_("Internal Error while copying to clipboard"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while copying to clipboard"),ex,false);
			}
    		}
    	}
    	/** Action class for "Selection Tool selected" */
	class SelectAction extends AbstractAction {
        	public SelectAction() {
            		super((String)_("Select Tool"), createIcon("Select"));
	        	putValue(SHORT_DESCRIPTION, _("Selects the Selection Tool"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{
            			miSelectedTool = IControl.SELECTION_TOOL;
	            		((ClXxy_childWnd)moClientArea.getSelectedFrame()).setTool(miSelectedTool);

	            		for (byte i=0;i<3;i++){
        	    			maToolButtons[i].setSelected(  (miSelectedTool==i)?true:false);
            				maToolMenuItems[i].setSelected((miSelectedTool==i)?true:false);
            			}
            		}catch(EXXYError ei){
				eMsg(_("Internal Error while setting the Selection Tool"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while setting the Selection Tool"),ex,false);
			}

        	}
    	}
    	/** Action class for "Create Label Tool selected" */
	class CreateLabelAction extends AbstractAction {
        	public CreateLabelAction() {
            		super((String)_("Label Tool"), createIcon("Label"));
	        	putValue(SHORT_DESCRIPTION, _("Selects the Label Creation Tool"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_L));
        	}
        	public void actionPerformed(ActionEvent e) {
			try{
				miSelectedTool = IControl.LABEL_CREATION_TOOL;
            			((ClXxy_childWnd)moClientArea.getSelectedFrame()).setTool(miSelectedTool);

            			for (byte i=0;i<3;i++){
            				maToolButtons[i].setSelected(  (miSelectedTool==i)?true:false);
            				maToolMenuItems[i].setSelected((miSelectedTool==i)?true:false);
            			}
            		}catch(EXXYError ei){
				eMsg(_("Internal Error while setting the Arrow Tool"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while setting the Arrow Tool"),ex,false);
			}

        	}
    	}
    	/** Action class for "Create Arrow Tool selected" */
	class CreateArrowAction extends AbstractAction {
        	public CreateArrowAction() {
            		super((String)_("Arrow Tool"), createIcon("Arrow"));
	        	putValue(SHORT_DESCRIPTION, _("Selects the Arrow Creation Tool"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_A));
        	}
        	public void actionPerformed(ActionEvent e) {
			try{
				miSelectedTool = IControl.ARROW_CREATION_TOOL;
	            		((ClXxy_childWnd)moClientArea.getSelectedFrame()).setTool(miSelectedTool);

	            		for (byte i=0;i<3;i++){
        	    			maToolButtons[i].setSelected(  (miSelectedTool==i)?true:false);
            				maToolMenuItems[i].setSelected((miSelectedTool==i)?true:false);
            			}
            		}catch(EXXYError ei){
				eMsg(_("Internal Error while setting the Label Tool"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while setting the Label Tool"),ex,false);
			}

        	}
    	}
    	/** Action class for "Delete Selected" */
	class DeleteAction extends AbstractAction {
        	public DeleteAction() {
            		super((String)_("Delete"), createIcon("Delete"));
	        	putValue(SHORT_DESCRIPTION, _("Deletes the currently selected Object"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_E));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{ ((ClXxy_childWnd)moClientArea.getSelectedFrame()).deleteSelected(); }
            		catch(EXXYError ei){
				eMsg(_("Internal Error while deleting Object"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while deleting Object"),ex,false);
			}
        	}
    	}

    	//Option Menu
    	/** Action class for "Show/Hide Grid" */
	class ShowGridAction extends AbstractAction {
        	public ShowGridAction() {
            		super((String)_("Show Grid"), createIcon("Grid"));
	        	putValue(SHORT_DESCRIPTION, _("Shows / Hides the Grid"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_G));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{
	            		boolean bIsVis = ((AbstractButton)e.getSource()).isSelected();
        	    		moGridVisibleButton.setSelected(bIsVis);
				moGridVisibleMenuItem.setSelected(bIsVis);
            			((ClXxy_childWnd)moClientArea.getSelectedFrame()).setGridVisible(bIsVis);
            		}catch(EXXYError ei){
				eMsg(_("Internal Error while hiding/showing grid"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while showing/hiding grid"),ex,false);
			}
        	}
    	}
    	//Help Menu
    	/** Action class for "Show about dialog" */
	class InfoAction extends AbstractAction {
        	public InfoAction() {
            		super((String)_("About..."), createIcon("About"));
	        	putValue(SHORT_DESCRIPTION, _("Displays the about Dialog"));
        		putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_B));
        	}
        	public void actionPerformed(ActionEvent e) {
            		try{ ClAboutDialog ad = new ClAboutDialog(mwTopWnd); }
            		catch(EXXYError ei){
				eMsg(_("Internal Error while showing the About Dialog"),ei,true);
			}
			catch(Exception ex){
				eMsg(_("Error while showing the About Dialog"),ex,false);
			}
        	}
    	}

}
