/** Defines the widget and all needed Listeners for editing selected Labels and arrows.
 *  Will be displayed at the bottom edge of the main frame, above the status bar.
 *
 *  @author Vitus Lorenz-Meyer
 *  @version 1.4 $Rev$
 */

package Control.Editor;
import Control.*;
import DAL.*;
import util.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ClEditorWidget extends JToolBar implements ActionListener,ISelChangedListener{
	/** the selected Object (Interface)
	 *  @see DAL.ILabel
	 *  @see DAL.IArrow */
	private Object mcSelectedObject;
	/** Type of selected Interface, as defined in PL.IPL
	 *  @see PL.IPL */
	private byte miSelType=-1;
	/** Main Frame (ClXxy), to send repaints to it */
	private ClXxy mwTopWnd;

	//root panel Objects
	/** root-panel for all Label related properties */
	private JPanel moLabelPanel = new JPanel();
	/** root-panel for all Arrow related properties */
	private JPanel moArrowPanel = new JPanel();
	/** root-panel if nothing is selected */
	private JLabel moNothingSelLabel = new JLabel(_("Nothing selected"));

	//Label panel Objects
	/** Caption of a Label */
	private JTextField mtLabelCaption = new JTextField(_("Caption"),20);

	//Arrow panel Objects
	//top
	/** Slider Checkbox of top Label */
	private JCheckBox moTopCheckBox = new JCheckBox();
	/** Alignment Slider of top Label */
	private JSlider moTopCaptionSlider;
	/** Caption of Top Label */
	private JTextField mtTopCaption = new JTextField(_("TopCaption"));

	//center
	/** Slider Checkbox of Center Label */
	private JCheckBox moCenterCheckBox = new JCheckBox();
	/** Alignment Slider of Center Label */
	private JSlider moCenterCaptionSlider;
	/** Caption of Center Label */
	private JTextField mtCenterCaption = new JTextField(_("CenterCaption"));

	//bottom
	/** Slider Checkbox of Bottom Label */
	private JCheckBox moBottomCheckBox = new JCheckBox();
	/** Alignment Slider of Bottom Label */
	private JSlider moBottomCaptionSlider;
	/** Caption of Bottom Label */
	private JTextField mtBottomCaption = new JTextField(_("BottomCaption"));

	/** Supported Types of an Arrow
	 *  @see DAL.IDiagram */
	private static String[] arrowTypes = {"->",".>","","="};
	/** Combo Box of Arrow Types*/
	JComboBox moArrowTypeBox = new JComboBox(arrowTypes);

	/** Array of all Checkboxes */
	JCheckBox[] moBoxes = {moTopCheckBox,moCenterCheckBox,moBottomCheckBox};
	/** Array of all Sliders */
	JSlider[] moSliders = {moTopCaptionSlider,moCenterCaptionSlider,moBottomCaptionSlider};
	/** Array of all Captions */
	JTextField[] moFields = {mtTopCaption,mtCenterCaption,mtBottomCaption};
	/** Prefix Strings */
	String[] mstrPrefixes = new String[3];
	/** convenience method */
	private static String _(String pstrGid){ return gnu.gettext.GettextResource.gettext(Control.ClXxy.moResBundle,pstrGid); }

	/** creates a new ClEditorWidget */
	public ClEditorWidget(ClXxy poXxy){
		mcSelectedObject = null;
		mwTopWnd = poXxy;
		mstrPrefixes[0] = _("top");
		mstrPrefixes[1] = _("center");
		mstrPrefixes[2] = _("bottom");

	//create the UI Components for the Properties
	//Label
		//mtLabelCaption.addActionListener(this);
		//mtLabelCaption.setActionCommand("Label");
		mtLabelCaption.getDocument().addDocumentListener(new DocumentListener(){
					public void insertUpdate(DocumentEvent e) {
        					try{ ((ILabel)mcSelectedObject).setCaption(mtLabelCaption.getText());}
        					catch(Exception ex){}
        					mwTopWnd.repaintCurrentDAL();
    					}
    					public void removeUpdate(DocumentEvent e) {
        					try{((ILabel)mcSelectedObject).setCaption(mtLabelCaption.getText());}
        					catch(Exception ex){}
        					mwTopWnd.repaintCurrentDAL();
    					}
    					public void changedUpdate(DocumentEvent e) {
    						mwTopWnd.repaintCurrentDAL();
        				}
    				}
    			);
		moLabelPanel.add(new JLabel(_("Caption")+":   "));
		moLabelPanel.add(mtLabelCaption);

	//Arrow
		//Type ComboBox
		moArrowTypeBox.setEditable(false);
		moArrowTypeBox.addActionListener(this);
		moArrowTypeBox.setActionCommand("Type");
		moArrowTypeBox.setPreferredSize(new Dimension(40,20));
		moArrowPanel.add(moArrowTypeBox);
		moArrowPanel.setBorder(BorderFactory.createEmptyBorder(0,10,0,10));

		//Label Properties (top,center,bottom)
		JPanel pp,p2;
		byte i;

		for (i=0; i<3; i++){
			pp = new JPanel(new BorderLayout());
			p2 = new JPanel();
			final byte p=i;
			moSliders[i] = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);

			//sizes
			moFields[i].setPreferredSize(new Dimension(100,20));
			moSliders[i].setPreferredSize(new Dimension(80,20));

			//action Listeners
			//moFields[i].setActionCommand(mstrPrefixes[i]+"_Caption");
			moBoxes[i].setActionCommand(mstrPrefixes[i]+"_Box");
			//Slider movment Listener
			moSliders[i].addChangeListener(
            			new ChangeListener() {
                			public void stateChanged(ChangeEvent e) {
	                    			if (miSelType==PL.IPL.ARROW)
	                    				((IArrow)mcSelectedObject).setAlignment(p,((int)((JSlider)e.getSource()).getValue()));
							mwTopWnd.repaintCurrentDAL();
        	            		}
            			}
        		);
			moBoxes[i].addActionListener(this);
			//moFields[i].addActionListener(this);
			moFields[i].getDocument().addDocumentListener(new DocumentListener(){
					public void insertUpdate(DocumentEvent e) {
        					try{ ((IArrow)mcSelectedObject).setCaption(p,moFields[p].getText());}
        					catch(Exception ex){}
        					mwTopWnd.repaintCurrentDAL();
    					}
    					public void removeUpdate(DocumentEvent e) {
        					try{ ((IArrow)mcSelectedObject).setCaption(p,moFields[p].getText());}
        					catch(Exception ex){}
        					mwTopWnd.repaintCurrentDAL();
    					}
    					public void changedUpdate(DocumentEvent e) {
    						mwTopWnd.repaintCurrentDAL();
        				}
    				}
    			);

			p2.add(moSliders[i]);
			p2.add(moBoxes[i]);
			pp.add(p2,BorderLayout.NORTH);
			pp.add(moFields[i],BorderLayout.CENTER);
			pp.setBorder(BorderFactory.createEmptyBorder(0,10,0,10));
			//add a short description and the panel with the slider on top and textfield at bottom to the ArrowPanel
			moArrowPanel.add(new JLabel(mstrPrefixes[i]+":  "));
			moArrowPanel.add(pp);
		}


	}
	/** the overloaded actionPerformed method (inherited from actionListener)
	 *  @see ActionListener
	 *  @param ActionEvent from the firing action */
	public void actionPerformed(ActionEvent e) {
 		String com = e.getActionCommand();

 		//if (com.equals("Label") && miSelType==PL.IPL.LABEL)		//Label verndern
		//	((ILabel)mcSelectedObject).setCaption(mtLabelCaption.getText());
		if (miSelType==PL.IPL.ARROW){
			if (com.equals("Type")){
				((IArrow)mcSelectedObject).setStyle(moArrowTypeBox.getSelectedIndex()+1);
				mwTopWnd.repaintCurrentDAL();
			}
			else for (byte i=0;i<3;i++){
				if (com.startsWith(mstrPrefixes[i])){
					if (com.endsWith("Box")){
						//set the arrow to unaligned if box is deselected
						if (!moBoxes[i].isSelected())
							((IArrow)mcSelectedObject).setAlignment(i,IArrow.ALIGNMENT_UNALIGNED);
						else    ((IArrow)mcSelectedObject).setAlignment(i,moSliders[i].getValue());
						//en-/disable the slider
						moSliders[i].setEnabled(moBoxes[i].isSelected());
						mwTopWnd.repaintCurrentDAL();
					}
				}
			}
		}
	}

	/** Method which gets called when a SelectionChanged event is fired (inherited from ISelChangedListener)
	 *  @see util.ISelChangedListener
	 *  @see PL.IPL
	 *  @param Interface to the DAL Object of the newly selected Item
	 *  @param Type of the newly selected Object (as defined in PL.IPL) */
	public void selectionChanged(Object poInterface, byte pType){

		setVisible(false);

		switch (pType){
				case PL.IPL.ARROW:
					if ( pType != miSelType){
						removeAll();
						add(moArrowPanel);
					}
					moArrowTypeBox.setSelectedIndex(((IArrow)poInterface).getStyle() - 1);
					int t;
					for (byte i=0;i<3;i++){
						moFields[i].setText(((IArrow)poInterface).getCaption(i));
								//unaligned
						if ( (t = (((IArrow)poInterface).getAlignment(i))) == IArrow.ALIGNMENT_UNALIGNED){
							moSliders[i].setEnabled(false);
							moBoxes[i].setSelected(false);
						}else{		//aligned
							moSliders[i].setValue(t);
							moSliders[i].setEnabled(true);
							moBoxes[i].setSelected(true);
						}
					}
					break;
				case PL.IPL.LABEL:
					if ( pType != miSelType){
						removeAll();
						add(moLabelPanel);
					}
					mtLabelCaption.setText(((ILabel)poInterface).getCaption());
					break;
				case PL.IPL.NOTHING:
					if ( pType != miSelType){
						removeAll();
						add(moNothingSelLabel);
					}
					break;
		}
		mcSelectedObject = poInterface;
		miSelType=pType;

		setVisible(true);

	}


}
