/** Interface that defines how the Data Abstraction Layer (DAL) represents an
 *  Arrow to the outside world.
 *
 *  @Author Philipp Kaluza
 *  @Version 1.0 $Rev: 244 $
 */
// $Id: IArrow.java 244 2003-05-25 15:15:01Z phk $

package DAL;

import util.*;

public interface IArrow{
	/** Constants to access arrow captions */
	final int CAPTION_POS_TOP	= 0;  // top arrow caption
	final int CAPTION_POS_MIDDLE	= 1;  // middle arrow caption
	final int CAPTION_POS_BOTTOM	= 2;  // bottom arrow caption

	/** Constants for caption alignment along the arrow */
	final int ALIGNMENT_UNALIGNED	= -1; // no special alignment

	/** Constants to define the style of the arrow shaft */
	final int STYLE_SHAFT_NORMAL	= 1;  // @{-}
	final int STYLE_SHAFT_DOTTED	= 2;  // @{.}
	final int STYLE_SHAFT_VOID	= 3;  // @{}
	final int STYLE_SHAFT_EQUALS	= 4;  // @2{-}


	/** get the coordinates of the arrow's base
	 *  @return the base's coordinates as ClCoord */
	ClCoord getBasePos();

	/** get the coordinates of the arrow's target
	 *  @return the target's coordinates as ClCoord */
	ClCoord getTargetPos();

	/** set a new base label for the arrow
	 *  @param pkNewPos the coordinates of the new base label */
	void	setBasePos(ClCoord pkNewPos) throws EXXYError;

	/** set a new target label for the arrow
	 *  @param pkNewPos the coordinates of the new target label */
	void	setTargetPos(ClCoord pkNewPos) throws EXXYError;


	/** get the top/middle/bottom caption of the arrow
	 *  @param  index the number of the caption as defined by CAPTION_POS_*
	 *  @return the target's coordinates as ClCoord */
	String	getCaption(int index);

	/** set the top/middle/bottom caption of the arrow
	 *  @param  index the number of the caption as defined by CAPTION_POS_*
	 *  @param  pstrCaption the new caption */
	void	setCaption(int index, String pstrCaption);


	/** get the top/middle/bottom caption's alignment along the arrow
	 *  @param  index the number of the caption as defined by CAPTION_POS_*
	 *  @return the alignment in percent */
	int	getAlignment(int index);

	/** set the top/middle/bottom caption's alignment along the arrow
	 *  @param  index the number of the caption as defined by CAPTION_POS_*
	 *  @param  piAlignment the new alignment in percent */
	void	setAlignment(int index, int piAlignment);


	/** get the style of the arrow
	 *  @return the style as defined by STYLE_SHAFT_* */
	int	getStyle();

	/** set the style of the arrow
	 *  @param piStyle the style as defined by STYLE_SHAFT_* */
	void	setStyle(int piStyle);


	/** unlink and destroy this arrow */
	void	remove();
}
