/** Interface that defines how the Data Abstraction Layer (DAL) represents a
 *  Diagram to the outside world.
 *
 *  @Author Philipp Kaluza
 *  @Version 1.0 $Rev: 244 $
 */
// $Id: IDiagram.java 244 2003-05-25 15:15:01Z phk $

package DAL;

import util.*;

public interface IDiagram{
	/** get the diagram's associated filename
	 *  @return the path filename */
	String	getFilename();

	/** set the diagram's associated filename
	 *  @param pstrFilename the new path filename */
	void	setFilename(String pstrFilename);


	/** create and add a new label to the diagram
	 *  @param pkPos the position to create the new label at */
	ILabel	addNewLabel(ClCoord pkPos) throws EXXYError;

	/** create and add a new arrow to the diagram
	 *  @param pkBase the base label's position
	 *  @param pkTarget the target label's position */
	IArrow	addNewArrow(ClCoord pkBase, ClCoord pkTarget) throws EXXYError;


	/** see if the diagram changed since it was last changed
	 *  @return the "dirty" status of the diagram */
	boolean	getDirty();

	/** tell the diagram it is "dirty" / "clean" (unsaved/saved)
	 *  @param pbDirty the current dirty status */
	void	setDirty(boolean pbDirty);


	/** return the label at the given position in the diagram
	 *  @param pkPos the position to inspect
	 *  @return the label there; or null if nonexistant */
	ILabel	getLabelAtPos(ClCoord pkPos);


	/** get all the labels in this diagram
	 *  @return an Iterator over these labels */
	java.util.Iterator	getLabelIterator();


	/** unlink and destroy this diagram (only in memory, ofcourse) */
	void	remove();
}
