/** Class that represents an Arrow as defined by IArrow
 *
 *  @Author Philipp Kaluza
 *  @Version 1.0 $Rev: 244 $
 */
// $Id: ClArrow.java 244 2003-05-25 15:15:01Z phk $

package DAL.Simple;

import util.*;
import DAL.*;

public class ClArrow implements DAL.IArrow {
	// Member variables:
	private ClDiagram	moParent;
	private ClLabel		moBase;
	private ClLabel		moTarget;

	private int		miStyle		= 0;

	private String[]	mastrCaptions	= {"", "", ""};
	private int[]		maiAlignments	= {50, 50, 50};

	public ClArrow(IDiagram poParent, ClCoord pkBase, ClCoord pkTarget) throws EXXYError {
		moParent = (ClDiagram) poParent;
		moBase   = Pos2Label(pkBase, true);
		moTarget = Pos2Label(pkTarget, true);

		moBase.regOrigArrow(this);
		moTarget.regEndArrow(this);
	}

	public ClCoord getBasePos() {
		return moBase.getPos();
	}

	public ClCoord getTargetPos() {
		return moTarget.getPos();
	}

	public void setBasePos(ClCoord pkNewPos) throws EXXYError {
		ClLabel l = Pos2Label(pkNewPos, false);
		moBase.unregOrigArrow(this);
		moBase = l;
		moBase.regOrigArrow(this);
	}

	public void setTargetPos(ClCoord pkNewPos) throws EXXYError {
		ClLabel l = Pos2Label(pkNewPos, false);
		moTarget.unregEndArrow(this);
		moTarget = l;
		moTarget.regEndArrow(this);
	}

	public String getCaption(int index) {
		return mastrCaptions[index];
	}

	public void setCaption(int index, String pstrCaption) {
		mastrCaptions[index] = pstrCaption;
	}

	public int getAlignment(int index) {
		return maiAlignments[index];
	}

	public void setAlignment(int index, int piAlignment) {
		maiAlignments[index] = piAlignment;
	}

	public int getStyle() {
		return miStyle;
	}

	public void setStyle(int piStyle) {
		miStyle = piStyle;
	}

	public void remove() {
		moBase.unregOrigArrow(this);
		moTarget.unregEndArrow(this);
		moBase   = null;
		moTarget = null;
	}

	///////////////////////////////////////////////////////////////////////

	private ClLabel Pos2Label(ClCoord pkPos, boolean pbCreate) throws EXXYError {
		ClLabel l = (ClLabel)moParent.getLabelAtPos(pkPos);
		if (l == null) {
			if (pbCreate) {
				l = new ClLabel(moParent,pkPos);
			} else {
				throw new ESyntacticError("No label there !");
			}
		}
		return l;
	}
}
