package PL;

import util.*;
import DAL.*;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

class ClArrow extends ClPLObj {
    private static final int SHAPE = 15;
    private static final int CAPTION_DIST = 30;
    private static final int CAPTION_BOXSIZE = 20;
    private static final double VERTEX_LENGTH = 15.0;
    private static final double VBOTTOM_LENGTH = 10.0;

    final static float mfDash1[ ] = {2.0f};
    final static BasicStroke moThin = new BasicStroke(1.0f);
    final static BasicStroke moNormal = new BasicStroke(2.0f);
    final static BasicStroke moDashed = new BasicStroke(3.0f,
                                                      BasicStroke.CAP_BUTT,
                                                      BasicStroke.JOIN_MITER,
                                                      2.0f, mfDash1, 0.0f);

    public void setTargetPos(ClCoord pkPos){
        try{
            ((IArrow)moDALObj).setTargetPos( pkPos );
        } catch( EXXYError e){
            System.out.println(e.toString());
        }        

    }

    
    public static ClCoord getBasePos(ClPLObj poArrow){
        ClCoord oBase = null;
        if(poArrow.ms8Type==IPL.ARROW){
            oBase = ((IArrow)poArrow.moDALObj).getBasePos();
        }
        return oBase;
    }

    public static ClCoord getTargetPos(ClPLObj poArrow){
        ClCoord oTarget = null;
        if(poArrow.ms8Type==IPL.ARROW){
            oTarget = ((IArrow)poArrow.moDALObj).getTargetPos();
        }
        return oTarget;
    }

    
    public static boolean touches(IArrow poArrow, ClPLObj poPLObj){
        ClCoord kLbl = ((ILabel) poPLObj.moDALObj).getPos(); 
        return ( kLbl.equals(poArrow.getBasePos()) ||
                   kLbl.equals(poArrow.getTargetPos()) );
    }
    
    public static boolean isInNodeArea(IArrow poArrow, Point poPos){
        boolean bArea = false;
        Point oBasePos = ClXyWidget.mat2pos( poArrow.getBasePos() );
        Point oTargetPos = ClXyWidget.mat2pos( poArrow.getTargetPos() );
        Point oStart = ClLabel.getLink2DPos(oBasePos, oTargetPos);
        Point oEnd = ClLabel.getLink2DPos(oTargetPos, oBasePos);
        Point vecArrow = new Point(oEnd.x-oStart.x, oEnd.y-oStart.y);
        double len = Math.sqrt(Math.pow(vecArrow.x, 2.0) + Math.pow(vecArrow.y, 2.0));
        int x = (int)(-vecArrow.y/len*SHAPE/2);
        int y = (int)(vecArrow.x/len*SHAPE/2);
        int[] xpoints = {oEnd.x+x,oEnd.x-x,oStart.x-x,oStart.x+x};
        int[] ypoints = {oEnd.y+y,oEnd.y-y,oStart.y-y,oStart.y+y};
        Polygon p = new Polygon(xpoints, ypoints, 4);
        return p.contains(poPos);
    }

    public static void paint(IArrow poArrow, Graphics poGraphic, boolean pbIsSelected){
        Point oBase = ClXyWidget.mat2pos( poArrow.getBasePos() );
        Point oTarget = ClXyWidget.mat2pos( poArrow.getTargetPos() );
        
        Point oStart = ClLabel.getLink2DPos(oBase, oTarget);
        Point oEnd = ClLabel.getLink2DPos(oTarget, oBase);
        
        Point vecArrow = new Point(oEnd.x-oStart.x, oEnd.y-oStart.y);
        /* length of vec */
        double len = Math.sqrt(Math.pow(vecArrow.x, 2.0) + Math.pow(vecArrow.y, 2.0));

        paintArrow(poGraphic, oStart, oEnd, vecArrow, len, poArrow.getStyle(), pbIsSelected);
 
        int[] nCapt = {poArrow.CAPTION_POS_TOP,
                        poArrow.CAPTION_POS_MIDDLE,
                        poArrow.CAPTION_POS_BOTTOM};
        
        Graphics2D oG2D = (Graphics2D) poGraphic;
        for(int i=0; i<nCapt.length; ++i){
            String strCaption = poArrow.getCaption(nCapt[i]);
            if(strCaption!=null && strCaption!=""){
                Point oCapt = new Point();
                int x = (int)(-vecArrow.y/len*CAPTION_DIST);
                int y = (int)(vecArrow.x/len*CAPTION_DIST);                
                /* get "startpos" */
                if(i==0){
                    oCapt.setLocation(oStart.x+x, oStart.y+y);
                }
                else if(i==1){
                    oCapt.setLocation(oStart.x, oStart.y);
                }
                else if(i==2){
                    oCapt.setLocation(oStart.x-x, oStart.y-y);
                }
                int nAlign = poArrow.getAlignment(nCapt[i]);
                /* add correct aligned pos */
                if(nAlign!=poArrow.ALIGNMENT_UNALIGNED){
                    oCapt.x += (vecArrow.x/100.0)*nAlign;
                    oCapt.y += (vecArrow.y/100.0)*nAlign;
                }
                else{
                    oCapt.x += vecArrow.x/2;
                    oCapt.y += vecArrow.y/2;
                }
                String strTxt = poArrow.getCaption(nCapt[i]);
                oG2D.setStroke(moThin);
                oG2D.setColor(Color.blue);
                if(strTxt.length()>3) strTxt = strTxt.substring(0, 2) + "...";
                oG2D.drawString(strTxt,oCapt.x-10, oCapt.y+5);
                oG2D.setColor(Color.gray);
                oG2D.drawRect(oCapt.x-CAPTION_BOXSIZE/2-4,
                              oCapt.y-CAPTION_BOXSIZE/2,
                              CAPTION_BOXSIZE+8,
                              CAPTION_BOXSIZE);
            }
        }
        
    }

    public static void paintOnMove(IArrow poArrow, Graphics poGraphic,
                                   ClPLObj oMovingObj,
                                   Point poActualPos,
                                   boolean pbIsSelected){
        Point oStart;
        Point oEnd;
        Point oBase = ClXyWidget.mat2pos( poArrow.getBasePos() );
        Point oTarget = ClXyWidget.mat2pos( poArrow.getTargetPos() );
        if(((ILabel)oMovingObj.moDALObj).getPos().equals( poArrow.getBasePos() )){
            oStart = ClLabel.getLink2DPos(poActualPos, oTarget);
            oEnd = ClLabel.getLink2DPos(oTarget, poActualPos);
        }
        else{
            oStart = ClLabel.getLink2DPos(oBase, poActualPos);
            oEnd = ClLabel.getLink2DPos(poActualPos, oBase);
        }
        Point vecArrow = new Point(oEnd.x-oStart.x, oEnd.y-oStart.y);
       /* length of vec **/
        double len = Math.sqrt(Math.pow(vecArrow.x, 2.0) + Math.pow(vecArrow.y, 2.0));
        paintArrow(poGraphic, oStart, oEnd, vecArrow, len, poArrow.getStyle(), pbIsSelected);
    }

    private static void paintArrow(Graphics poGraphic,
                                Point oStart, 
                                Point oEnd,
                                Point pvecArrow,
                                double pArrLength,
                                int pnStyle,
                                boolean pbIsSelected){
        /* orthogonal vec with length VBOTTOM_LENGTH */
        int x = (int)(-pvecArrow.y/pArrLength*VBOTTOM_LENGTH/2);
        int y = (int)(pvecArrow.x/pArrLength*VBOTTOM_LENGTH/2);
        /* middle of vertex */
        int vmxEnd = oEnd.x - (int)(pvecArrow.x/pArrLength*VERTEX_LENGTH);
        int vmyEnd = oEnd.y - (int)(pvecArrow.y/pArrLength*VERTEX_LENGTH);

        Graphics2D oG2D = (Graphics2D) poGraphic;
        oG2D.setStroke(moNormal);
        
        if(pbIsSelected) oG2D.setColor(moSelColor);
        else oG2D.setColor(Color.darkGray);

        switch(pnStyle){
            case IArrow.STYLE_SHAFT_NORMAL:
                break;
            case IArrow.STYLE_SHAFT_DOTTED:
                oG2D.setStroke(moDashed); break;
            case IArrow.STYLE_SHAFT_VOID:
                if(!pbIsSelected) oG2D.setColor(Color.lightGray); break;
            case IArrow.STYLE_SHAFT_EQUALS:
                oG2D.setStroke(moThin);
                int vmxStart = oStart.x + (int)(pvecArrow.x/pArrLength*VERTEX_LENGTH);
                int vmyStart = oStart.y + (int)(pvecArrow.y/pArrLength*VERTEX_LENGTH);
                int[] xPoints = {vmxStart+x, vmxStart-x, oStart.x};
                int[] yPoints = {vmyStart+y, vmyStart-y, oStart.y};
                oG2D.drawPolygon(xPoints, yPoints, 3);
                oG2D.fillPolygon(xPoints, yPoints, 3);
                int x1 = (int)(-pvecArrow.y/pArrLength*3);
                int y1 = (int)(pvecArrow.x/pArrLength*3);
                oG2D.draw(new Line2D.Double(vmxStart+x1, vmyStart+y1, vmxEnd+x1, vmyEnd+y1));
                oG2D.draw(new Line2D.Double(vmxStart-x1, vmyStart-y1, vmxEnd-x1, vmyEnd-y1));
                break;
        }
        vmxEnd = oEnd.x - (int)(pvecArrow.x/pArrLength*VERTEX_LENGTH);
        vmyEnd = oEnd.y - (int)(pvecArrow.y/pArrLength*VERTEX_LENGTH);
        int[] xPoints = {vmxEnd+x, vmxEnd-x, oEnd.x};
        int[] yPoints = {vmyEnd+y, vmyEnd-y, oEnd.y};

        if(pnStyle!=IArrow.STYLE_SHAFT_EQUALS){
            oG2D.draw(new Line2D.Double(oStart.x, oStart.y, vmxEnd, vmyEnd));
        }
        oG2D.setStroke(moNormal);
        oG2D.drawPolygon(xPoints, yPoints, 3);
        oG2D.fillPolygon(xPoints, yPoints, 3);
       // oG2D.drawLine(oStart.x, oStart.y, oEnd.x, oEnd.y);
    }


    public static void paintOnMoveNewArrow(Graphics poGraph, ClCoord pkBase,
                                    Point poEnd, Color oColor){
        poGraph.setColor(oColor);
        Point oBasePos = ClXyWidget.mat2pos( pkBase );
        Point oStart = ClLabel.getLink2DPos(oBasePos, poEnd);
        poGraph.drawLine(oStart.x, oStart.y, poEnd.x, poEnd.y);
    }
}

