/** IMMUTABLE class that represents a logical coordinate pair within a diagram
 *
 *  @Author Philipp Kaluza
 *  @Version 1.0 $Rev: 241 $
 */
// $Id: ClCoord.java 241 2003-05-21 15:50:26Z snoopy $

package util;

// this class is /final/ to enforce it's immutability
public final class ClCoord {
	// since this is an immutable class, these members don't need to change
	private final int miX;
	private final int miY;

	/** constructor to create a new ClCoord instance
	 *  @param iX the x-coordinate
	 *  @param iY the x-coordinate */
	public ClCoord(int piX, int piY) {
		miX = piX;
		miY = piY;
	}

	/** constructor to create a new ClCoord instance from another one
	 *  (probably obsolete)
	 *  @param cAncestor the ClCoord to clone */
	public ClCoord(ClCoord cAncestor) {
		miX = cAncestor.getX();
		miY = cAncestor.getY();
	}


	/** compare two ClCoords
	 *  @return true==equal */
	public final boolean equals(ClCoord pkRight) {
                boolean bIsEqual = true;
                if(this!=pkRight){
                    if( miX != pkRight.miX ) bIsEqual = false;
                    else if( miY != pkRight.miY ) bIsEqual = false;
                }
                return bIsEqual;
	}


	/** get the CLCoord's x-component
	 *  @return the x coordinate */
	public final int getX() {
		return miX;
	}

	/** get the CLCoord's y-component
	 *  @return the y coordinate */
	public final int getY() {
		return miY;
	}



	/** output ClCoord as String
	 *  @return the x- and y-coordinate as human-readable string */
	public final String toString() {
		return "(" + miX + "," + miY + ")";
	}
}
